/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

// README.txt generation entry point.
const $generatorReadme = {};

$generatorReadme.generatedBy = function (res) {
    res.line('Content of this folder was generated by Apache Ignite Web Console');
    res.line('=================================================================');

    res.needEmptyLine = true;
};

/**
 * Generate README.txt.
 *
 * @param res Resulting output with generated readme.
 * @returns {string} Generated content.
 */
$generatorReadme.readme = function (res) {
    if (!res)
        res = $generatorCommon.builder();

    $generatorReadme.generatedBy(res);

    res.line('Project structure:');
    res.line('    /config - this folder contains client and server XML configurations.');
    res.line('    /jdbc-drivers - this folder should contains proprietary JDBC drivers.');
    res.line('    /src - this folder contains generated java code.');
    res.line('    /src/main/java/config - this folder contains generated java classes with cluster configuration from code.');
    res.line('    /src/main/java/startup - this folder contains generated java classes with server and client nodes startup code.');
    res.line('    /src/main/java/[model] - this optional folder will be named as package name for your POJO classes and contain generated POJO files.');
    res.line('    /src/main/resources - this optional folder contains generated secret.properties file with security sensitive information if any.');
    res.line('    Dockerfile - sample Docker file. With this file you could package Ignite deployment with all the dependencies into a standard container.');
    res.line('    pom.xml - generated Maven project description, could be used to open generated project in IDE or build with Maven.');
    res.line('    README.txt - this file.');

    res.needEmptyLine = true;

    res.line('Ignite ships with CacheJdbcPojoStore, which is out-of-the-box JDBC implementation of the IgniteCacheStore ');
    res.line('interface, and automatically handles all the write-through and read-through logic.');

    res.needEmptyLine = true;

    res.line('You can use generated configuration and POJO classes as part of your application.');

    res.needEmptyLine = true;

    res.line('Note, in case of using proprietary JDBC drivers (Oracle, IBM DB2, Microsoft SQL Server)');
    res.line('you should download them manually and copy into ./jdbc-drivers folder.');

    return res;
};

/**
 * Generate README.txt for jdbc folder.
 *
 * @param res Resulting output with generated readme.
 * @returns {string} Generated content.
 */
$generatorReadme.readmeJdbc = function (res) {
    if (!res)
        res = $generatorCommon.builder();

    res.line('Proprietary JDBC drivers for databases like Oracle, IBM DB2, Microsoft SQL Server are not available on Maven Central repository.');
    res.line('Drivers should be downloaded manually and copied to this folder.');

    return res;
};

export default $generatorReadme;
